@@ -3,7 +3,7 @@
  * RemoteFX Codec Library - Encode
  *
  * Copyright 2011 Vic Lee
- * Copyright 2014-2015 Jay Sorg <jay.sorg@gmail.com>
+ * Copyright 2014-2017 Jay Sorg <jay.sorg@gmail.com>
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -18,6 +18,10 @@
  * limitations under the License.
  */
 
+#if defined(HAVE_CONFIG_H)
+#include <config_ac.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -49,7 +53,7 @@
 
 /******************************************************************************/
 static int
-rfx_encode_format_rgb(char *rgb_data, int width, int height,
+rfx_encode_format_rgb(const char *rgb_data, int width, int height,
                       int stride_bytes, int pixel_format,
                       uint8 *r_buf, uint8 *g_buf, uint8 *b_buf)
 {
@@ -72,7 +76,7 @@ rfx_encode_format_rgb(char *rgb_data, int width, int height,
         case RFX_FORMAT_BGRA:
             for (y = 0; y < height; y++)
             {
-                src = (uint8*) (rgb_data + y * stride_bytes);
+                src = (uint8 *) (rgb_data + y * stride_bytes);
                 lr_buf = r_buf + y * 64;
                 lg_buf = g_buf + y * 64;
                 lb_buf = b_buf + y * 64;
@@ -108,7 +112,7 @@ rfx_encode_format_rgb(char *rgb_data, int width, int height,
         case RFX_FORMAT_RGBA:
             for (y = 0; y < height; y++)
             {
-                src = (uint8*) (rgb_data + y * stride_bytes);
+                src = (uint8 *) (rgb_data + y * stride_bytes);
                 lr_buf = r_buf + y * 64;
                 lg_buf = g_buf + y * 64;
                 lb_buf = b_buf + y * 64;
@@ -144,7 +148,7 @@ rfx_encode_format_rgb(char *rgb_data, int width, int height,
         case RFX_FORMAT_BGR:
             for (y = 0; y < height; y++)
             {
-                src = (uint8*) (rgb_data + y * stride_bytes);
+                src = (uint8 *) (rgb_data + y * stride_bytes);
                 lr_buf = r_buf + y * 64;
                 lg_buf = g_buf + y * 64;
                 lb_buf = b_buf + y * 64;
@@ -179,7 +183,7 @@ rfx_encode_format_rgb(char *rgb_data, int width, int height,
         case RFX_FORMAT_RGB:
             for (y = 0; y < height; y++)
             {
-                src = (uint8*) (rgb_data + y * stride_bytes);
+                src = (uint8 *) (rgb_data + y * stride_bytes);
                 lr_buf = r_buf + y * 64;
                 lg_buf = g_buf + y * 64;
                 lb_buf = b_buf + y * 64;
@@ -217,7 +221,7 @@ rfx_encode_format_rgb(char *rgb_data, int width, int height,
 
 /******************************************************************************/
 static int
-rfx_encode_format_argb(char *argb_data, int width, int height,
+rfx_encode_format_argb(const char *argb_data, int width, int height,
                        int stride_bytes, int pixel_format,
                        uint8 *a_buf, uint8 *r_buf, uint8 *g_buf, uint8 *b_buf)
 {
@@ -243,7 +247,7 @@ rfx_encode_format_argb(char *argb_data, int width, int height,
         case RFX_FORMAT_BGRA:
             for (y = 0; y < height; y++)
             {
-                src = (uint8*) (argb_data + y * stride_bytes);
+                src = (uint8 *) (argb_data + y * stride_bytes);
                 la_buf = a_buf + y * 64;
                 lr_buf = r_buf + y * 64;
                 lg_buf = g_buf + y * 64;
@@ -284,7 +288,7 @@ rfx_encode_format_argb(char *argb_data, int width, int height,
         case RFX_FORMAT_RGBA:
             for (y = 0; y < height; y++)
             {
-                src = (uint8*) (argb_data + y * stride_bytes);
+                src = (uint8 *) (argb_data + y * stride_bytes);
                 la_buf = a_buf + y * 64;
                 lr_buf = r_buf + y * 64;
                 lg_buf = g_buf + y * 64;
@@ -325,7 +329,7 @@ rfx_encode_format_argb(char *argb_data, int width, int height,
         case RFX_FORMAT_BGR:
             for (y = 0; y < height; y++)
             {
-                src = (uint8*) (argb_data + y * stride_bytes);
+                src = (uint8 *) (argb_data + y * stride_bytes);
                 lr_buf = r_buf + y * 64;
                 lg_buf = g_buf + y * 64;
                 lb_buf = b_buf + y * 64;
@@ -360,7 +364,7 @@ rfx_encode_format_argb(char *argb_data, int width, int height,
         case RFX_FORMAT_RGB:
             for (y = 0; y < height; y++)
             {
-                src = (uint8*) (argb_data + y * stride_bytes);
+                src = (uint8 *) (argb_data + y * stride_bytes);
                 lr_buf = r_buf + y * 64;
                 lg_buf = g_buf + y * 64;
                 lb_buf = b_buf + y * 64;
@@ -435,7 +439,8 @@ rfx_encode_rgb_to_yuv(uint8 *y_r_buf, uint8 *u_g_buf, uint8 *v_b_buf)
 /******************************************************************************/
 int
 rfx_encode_component_rlgr1(struct rfxencode *enc, const char *qtable,
-                           uint8 *data, uint8 *buffer, int buffer_size, int *size)
+                           const uint8 *data,
+                           uint8 *buffer, int buffer_size, int *size)
 {
     LLOGLN(10, ("rfx_encode_component_rlgr1:"));
     if (rfx_dwt_2d_encode(data, enc->dwt_buffer1, enc->dwt_buffer) != 0)
@@ -457,7 +462,8 @@ rfx_encode_component_rlgr1(struct rfxencode *enc, const char *qtable,
 /******************************************************************************/
 int
 rfx_encode_component_rlgr3(struct rfxencode *enc, const char *qtable,
-                           uint8 *data, uint8 *buffer, int buffer_size, int *size)
+                           const uint8 *data,
+                           uint8 *buffer, int buffer_size, int *size)
 {
     LLOGLN(10, ("rfx_encode_component_rlgr3:"));
     if (rfx_dwt_2d_encode(data, enc->dwt_buffer1, enc->dwt_buffer) != 0)
@@ -478,199 +484,7 @@ rfx_encode_component_rlgr3(struct rfxencode *enc, const char *qtable,
 
 /******************************************************************************/
 int
-rfx_encode_component_rlgr1_x86_sse2(struct rfxencode *enc, const char *qtable,
-                                    uint8 *data,
-                                    uint8 *buffer, int buffer_size, int *size)
-{
-    LLOGLN(10, ("rfx_encode_component_rlgr1_x86_sse2:"));
-#if defined(RFX_USE_ACCEL_X86)
-    if (rfxcodec_encode_dwt_shift_x86_sse2(qtable, data, enc->dwt_buffer1,
-                                           enc->dwt_buffer) != 0)
-    {
-        return 1;
-    }
-    //*size = rfxcodec_encode_diff_rlgr1_x86_sse2(enc->dwt_buffer1,
-    //                                            buffer, buffer_size);
-    if (rfx_differential_encode(enc->dwt_buffer1 + 4032, 64) != 0)
-    {
-        return 1;
-    }
-    *size = rfx_rlgr1_encode(enc->dwt_buffer1, buffer, buffer_size);
-#endif
-    return 0;
-}
-
-/******************************************************************************/
-int
-rfx_encode_component_rlgr3_x86_sse2(struct rfxencode *enc, const char *qtable,
-                                    uint8 *data,
-                                    uint8 *buffer, int buffer_size, int *size)
-{
-    LLOGLN(10, ("rfx_encode_component_rlgr3_x86_sse2:"));
-#if defined(RFX_USE_ACCEL_X86)
-    if (rfxcodec_encode_dwt_shift_x86_sse2(qtable, data, enc->dwt_buffer1,
-                                           enc->dwt_buffer) != 0)
-    {
-        return 1;
-    }
-    //*size = rfxcodec_encode_diff_rlgr3_x86_sse2(enc->dwt_buffer1,
-    //                                            buffer, buffer_size);
-    if (rfx_differential_encode(enc->dwt_buffer1 + 4032, 64) != 0)
-    {
-        return 1;
-    }
-    *size = rfx_rlgr3_encode(enc->dwt_buffer1, buffer, buffer_size);
-#endif
-    return 0;
-}
-
-/******************************************************************************/
-int
-rfx_encode_component_rlgr1_x86_sse41(struct rfxencode *enc, const char *qtable,
-                                     uint8 *data,
-                                     uint8 *buffer, int buffer_size, int *size)
-{
-    LLOGLN(10, ("rfx_encode_component_rlgr1_x86_sse41:"));
-#if defined(RFX_USE_ACCEL_X86)
-    if (rfxcodec_encode_dwt_shift_x86_sse41(qtable, data, enc->dwt_buffer1,
-                                            enc->dwt_buffer) != 0)
-    {
-        return 1;
-    }
-    //*size = rfxcodec_encode_diff_rlgr1_x86_sse2(enc->dwt_buffer1,
-    //                                            buffer, buffer_size);
-    if (rfx_differential_encode(enc->dwt_buffer1 + 4032, 64) != 0)
-    {
-        return 1;
-    }
-    *size = rfx_rlgr1_encode(enc->dwt_buffer1, buffer, buffer_size);
-#endif
-    return 0;
-}
-
-/******************************************************************************/
-int
-rfx_encode_component_rlgr3_x86_sse41(struct rfxencode *enc, const char *qtable,
-                                     uint8 *data,
-                                     uint8 *buffer, int buffer_size, int *size)
-{
-    LLOGLN(10, ("rfx_encode_component_rlgr3_x86_sse41:"));
-#if defined(RFX_USE_ACCEL_X86)
-    if (rfxcodec_encode_dwt_shift_x86_sse41(qtable, data, enc->dwt_buffer1,
-                                            enc->dwt_buffer) != 0)
-    {
-        return 1;
-    }
-    //*size = rfxcodec_encode_diff_rlgr3_x86_sse(enc->dwt_buffer1,
-    //                                            buffer, buffer_size);
-    if (rfx_differential_encode(enc->dwt_buffer1 + 4032, 64) != 0)
-    {
-        return 1;
-    }
-    *size = rfx_rlgr3_encode(enc->dwt_buffer1, buffer, buffer_size);
-#endif
-    return 0;
-}
-
-/******************************************************************************/
-int
-rfx_encode_component_rlgr1_amd64_sse2(struct rfxencode *enc, const char *qtable,
-                                      uint8 *data,
-                                      uint8 *buffer, int buffer_size, int *size)
-{
-    LLOGLN(10, ("rfx_encode_component_rlgr1_amd64_sse2:"));
-#if defined(RFX_USE_ACCEL_AMD64)
-    if (rfxcodec_encode_dwt_shift_amd64_sse2(qtable, data, enc->dwt_buffer1,
-                                             enc->dwt_buffer) != 0)
-    {
-        return 1;
-    }
-    //*size = rfxcodec_encode_diff_rlgr1_amd64_sse2(enc->dwt_buffer1,
-    //                                              buffer, buffer_size);
-    if (rfx_differential_encode(enc->dwt_buffer1 + 4032, 64) != 0)
-    {
-        return 1;
-    }
-    *size = rfx_rlgr1_encode(enc->dwt_buffer1, buffer, buffer_size);
-#endif
-    return 0;
-}
-
-/******************************************************************************/
-int
-rfx_encode_component_rlgr3_amd64_sse2(struct rfxencode *enc, const char *qtable,
-                                      uint8 *data,
-                                      uint8 *buffer, int buffer_size, int *size)
-{
-    LLOGLN(10, ("rfx_encode_component_rlgr3_amd64_sse2:"));
-#if defined(RFX_USE_ACCEL_AMD64)
-    if (rfxcodec_encode_dwt_shift_amd64_sse2(qtable, data, enc->dwt_buffer1,
-                                             enc->dwt_buffer) != 0)
-    {
-        return 1;
-    }
-    //*size = rfxcodec_encode_diff_rlgr3_amd64_sse2(enc->dwt_buffer1,
-    //                                              buffer, buffer_size);
-    if (rfx_differential_encode(enc->dwt_buffer1 + 4032, 64) != 0)
-    {
-        return 1;
-    }
-    *size = rfx_rlgr3_encode(enc->dwt_buffer1, buffer, buffer_size);
-#endif
-    return 0;
-}
-
-/******************************************************************************/
-int
-rfx_encode_component_rlgr1_amd64_sse41(struct rfxencode *enc, const char *qtable,
-                                       uint8 *data,
-                                       uint8 *buffer, int buffer_size, int *size)
-{
-    LLOGLN(10, ("rfx_encode_component_rlgr1_amd64_sse2:"));
-#if defined(RFX_USE_ACCEL_AMD64)
-    if (rfxcodec_encode_dwt_shift_amd64_sse41(qtable, data, enc->dwt_buffer1,
-                                              enc->dwt_buffer) != 0)
-    {
-        return 1;
-    }
-    //*size = rfxcodec_encode_diff_rlgr1_amd64_sse2(enc->dwt_buffer1,
-    //                                              buffer, buffer_size);
-    if (rfx_differential_encode(enc->dwt_buffer1 + 4032, 64) != 0)
-    {
-        return 1;
-    }
-    *size = rfx_rlgr1_encode(enc->dwt_buffer1, buffer, buffer_size);
-#endif
-    return 0;
-}
-
-/******************************************************************************/
-int
-rfx_encode_component_rlgr3_amd64_sse41(struct rfxencode *enc, const char *qtable,
-                                       uint8 *data,
-                                       uint8 *buffer, int buffer_size, int *size)
-{
-    LLOGLN(10, ("rfx_encode_component_rlgr3_amd64_sse2:"));
-#if defined(RFX_USE_ACCEL_AMD64)
-    if (rfxcodec_encode_dwt_shift_amd64_sse41(qtable, data, enc->dwt_buffer1,
-                                              enc->dwt_buffer) != 0)
-    {
-        return 1;
-    }
-    //*size = rfxcodec_encode_diff_rlgr3_amd64_sse2(enc->dwt_buffer1,
-    //                                              buffer, buffer_size);
-    if (rfx_differential_encode(enc->dwt_buffer1 + 4032, 64) != 0)
-    {
-        return 1;
-    }
-    *size = rfx_rlgr3_encode(enc->dwt_buffer1, buffer, buffer_size);
-#endif
-    return 0;
-}
-
-/******************************************************************************/
-int
-rfx_encode_rgb(struct rfxencode *enc, char *rgb_data,
+rfx_encode_rgb(struct rfxencode *enc, const char *rgb_data,
                int width, int height, int stride_bytes,
                const char *y_quants, const char *u_quants,
                const char *v_quants,
@@ -725,7 +539,7 @@ rfx_encode_rgb(struct rfxencode *enc, char *rgb_data,
 
 /******************************************************************************/
 int
-rfx_encode_argb(struct rfxencode *enc, char *rgb_data,
+rfx_encode_argb(struct rfxencode *enc, const char *rgb_data,
                 int width, int height, int stride_bytes,
                 const char *y_quants, const char *u_quants,
                 const char *v_quants,
@@ -786,19 +600,19 @@ rfx_encode_argb(struct rfxencode *enc, char *rgb_data,
 
 /******************************************************************************/
 int
-rfx_encode_yuv(struct rfxencode *enc, char *yuv_data,
+rfx_encode_yuv(struct rfxencode *enc, const char *yuv_data,
                int width, int height, int stride_bytes,
                const char *y_quants, const char *u_quants,
                const char *v_quants,
                STREAM *data_out, int *y_size, int *u_size, int *v_size)
 {
-    uint8 *y_buffer;
-    uint8 *u_buffer;
-    uint8 *v_buffer;
+    const uint8 *y_buffer;
+    const uint8 *u_buffer;
+    const uint8 *v_buffer;
 
-    y_buffer = (uint8 *) yuv_data;
-    u_buffer = (uint8 *) (yuv_data + RFX_YUV_BTES);
-    v_buffer = (uint8 *) (yuv_data + RFX_YUV_BTES * 2);
+    y_buffer = (const uint8 *) yuv_data;
+    u_buffer = (const uint8 *) (yuv_data + RFX_YUV_BTES);
+    v_buffer = (const uint8 *) (yuv_data + RFX_YUV_BTES * 2);
     if (enc->rfx_encode(enc, y_quants, y_buffer,
                         stream_get_tail(data_out),
                         stream_get_left(data_out),
@@ -828,22 +642,22 @@ rfx_encode_yuv(struct rfxencode *enc, char *yuv_data,
 
 /******************************************************************************/
 int
-rfx_encode_yuva(struct rfxencode *enc, char *yuva_data,
+rfx_encode_yuva(struct rfxencode *enc, const char *yuva_data,
                 int width, int height, int stride_bytes,
                 const char *y_quants, const char *u_quants,
                 const char *v_quants,
                 STREAM *data_out, int *y_size, int *u_size,
                 int *v_size, int *a_size)
 {
-    uint8 *y_buffer;
-    uint8 *u_buffer;
-    uint8 *v_buffer;
-    uint8 *a_buffer;
+    const uint8 *y_buffer;
+    const uint8 *u_buffer;
+    const uint8 *v_buffer;
+    const uint8 *a_buffer;
 
-    y_buffer = (uint8 *) yuva_data;
-    u_buffer = (uint8 *) (yuva_data + RFX_YUV_BTES);
-    v_buffer = (uint8 *) (yuva_data + RFX_YUV_BTES * 2);
-    a_buffer = (uint8 *) (yuva_data + RFX_YUV_BTES * 3);
+    y_buffer = (const uint8 *) yuva_data;
+    u_buffer = (const uint8 *) (yuva_data + RFX_YUV_BTES);
+    v_buffer = (const uint8 *) (yuva_data + RFX_YUV_BTES * 2);
+    a_buffer = (const uint8 *) (yuva_data + RFX_YUV_BTES * 3);
     if (enc->rfx_encode(enc, y_quants, y_buffer,
                         stream_get_tail(data_out),
                         stream_get_left(data_out),
