@@ -19,10 +19,14 @@
  * RDP Capability Sets
  */
 
+#if defined(HAVE_CONFIG_H)
+#include <config_ac.h>
+#endif
+
 #include "libxrdp.h"
 
 /*****************************************************************************/
-static int APP_CC
+static int
 xrdp_caps_send_monitorlayout(struct xrdp_rdp *self)
 {
     struct stream *s;
@@ -62,7 +66,7 @@ xrdp_caps_send_monitorlayout(struct xrdp_rdp *self)
 }
 
 /*****************************************************************************/
-static int APP_CC
+static int
 xrdp_caps_process_general(struct xrdp_rdp *self, struct stream *s,
                           int len)
 {
@@ -97,7 +101,7 @@ xrdp_caps_process_general(struct xrdp_rdp *self, struct stream *s,
 }
 
 /*****************************************************************************/
-static int APP_CC
+static int
 xrdp_caps_process_order(struct xrdp_rdp *self, struct stream *s,
                         int len)
 {
@@ -174,7 +178,7 @@ xrdp_caps_process_order(struct xrdp_rdp *self, struct stream *s,
 
 /*****************************************************************************/
 /* get the bitmap cache size */
-static int APP_CC
+static int
 xrdp_caps_process_bmpcache(struct xrdp_rdp *self, struct stream *s,
                            int len)
 {
@@ -216,7 +220,7 @@ xrdp_caps_process_bmpcache(struct xrdp_rdp *self, struct stream *s,
 
 /*****************************************************************************/
 /* get the bitmap cache size */
-static int APP_CC
+static int
 xrdp_caps_process_bmpcache2(struct xrdp_rdp *self, struct stream *s,
                             int len)
 {
@@ -259,7 +263,7 @@ xrdp_caps_process_bmpcache2(struct xrdp_rdp *self, struct stream *s,
 }
 
 /*****************************************************************************/
-static int APP_CC
+static int
 xrdp_caps_process_cache_v3_codec_id(struct xrdp_rdp *self, struct stream *s,
                                     int len)
 {
@@ -279,7 +283,7 @@ xrdp_caps_process_cache_v3_codec_id(struct xrdp_rdp *self, struct stream *s,
 
 /*****************************************************************************/
 /* get the number of client cursor cache */
-static int APP_CC
+static int
 xrdp_caps_process_pointer(struct xrdp_rdp *self, struct stream *s,
                           int len)
 {
@@ -321,7 +325,7 @@ xrdp_caps_process_pointer(struct xrdp_rdp *self, struct stream *s,
 }
 
 /*****************************************************************************/
-static int APP_CC
+static int
 xrdp_caps_process_input(struct xrdp_rdp *self, struct stream *s,
                         int len)
 {
@@ -340,7 +344,7 @@ xrdp_caps_process_input(struct xrdp_rdp *self, struct stream *s,
 
 /*****************************************************************************/
 /* get the type of client brush cache */
-int APP_CC
+int
 xrdp_caps_process_brushcache(struct xrdp_rdp *self, struct stream *s,
                              int len)
 {
@@ -357,7 +361,7 @@ xrdp_caps_process_brushcache(struct xrdp_rdp *self, struct stream *s,
 }
 
 /*****************************************************************************/
-int APP_CC
+int
 xrdp_caps_process_offscreen_bmpcache(struct xrdp_rdp *self, struct stream *s,
                                      int len)
 {
@@ -383,7 +387,7 @@ xrdp_caps_process_offscreen_bmpcache(struct xrdp_rdp *self, struct stream *s,
 }
 
 /*****************************************************************************/
-int APP_CC
+int
 xrdp_caps_process_rail(struct xrdp_rdp *self, struct stream *s, int len)
 {
     int i32;
@@ -401,7 +405,7 @@ xrdp_caps_process_rail(struct xrdp_rdp *self, struct stream *s, int len)
 }
 
 /*****************************************************************************/
-static int APP_CC
+static int
 xrdp_caps_process_window(struct xrdp_rdp *self, struct stream *s, int len)
 {
     int i32;
@@ -426,7 +430,7 @@ xrdp_caps_process_window(struct xrdp_rdp *self, struct stream *s, int len)
 }
 
 /*****************************************************************************/
-static int APP_CC
+static int
 xrdp_caps_process_codecs(struct xrdp_rdp *self, struct stream *s, int len)
 {
     int codec_id;
@@ -521,7 +525,7 @@ xrdp_caps_process_codecs(struct xrdp_rdp *self, struct stream *s, int len)
 }
 
 /*****************************************************************************/
-static int APP_CC
+static int
 xrdp_caps_process_multifragmentupdate(struct xrdp_rdp *self, struct stream *s,
                                       int len)
 {
@@ -533,18 +537,24 @@ xrdp_caps_process_multifragmentupdate(struct xrdp_rdp *self, struct stream *s,
 }
 
  /*****************************************************************************/
-static int APP_CC
+static int
 xrdp_caps_process_frame_ack(struct xrdp_rdp *self, struct stream *s, int len)
 {
     g_writeln("xrdp_caps_process_frame_ack:");
     self->client_info.use_frame_acks = 1;
     in_uint32_le(s, self->client_info.max_unacknowledged_frame_count);
+    if (self->client_info.max_unacknowledged_frame_count < 0)
+    {
+        g_writeln("  invalid max_unacknowledged_frame_count value (%d), setting to 0",
+                  self->client_info.max_unacknowledged_frame_count);
+        self->client_info.max_unacknowledged_frame_count = 0;
+    }
     g_writeln("  max_unacknowledged_frame_count %d", self->client_info.max_unacknowledged_frame_count);
     return 0;
 }
 
 /*****************************************************************************/
-static int APP_CC
+static int
 xrdp_caps_process_surface_cmds(struct xrdp_rdp *self, struct stream *s, int len)
 {
     int cmdFlags;
@@ -556,7 +566,7 @@ xrdp_caps_process_surface_cmds(struct xrdp_rdp *self, struct stream *s, int len)
 }
 
 /*****************************************************************************/
-int APP_CC
+int
 xrdp_caps_process_confirm_active(struct xrdp_rdp *self, struct stream *s)
 {
     int cap_len;
@@ -707,7 +717,7 @@ xrdp_caps_process_confirm_active(struct xrdp_rdp *self, struct stream *s)
     return 0;
 }
 /*****************************************************************************/
-int APP_CC
+int
 xrdp_caps_send_demand_active(struct xrdp_rdp *self)
 {
     struct stream *s;
