@@ -17,6 +17,10 @@
  * limitations under the License.
  */
 
+#if defined(HAVE_CONFIG_H)
+#include <config_ac.h>
+#endif
+
 #include "xrdp-neutrinordp.h"
 #include "xrdp-color.h"
 #include "xrdp_rail.h"
@@ -60,7 +64,7 @@ verifyColorMap(struct mod *mod)
 
 /*****************************************************************************/
 /* return error */
-static int DEFAULT_CC
+static int
 lxrdp_start(struct mod *mod, int w, int h, int bpp)
 {
     rdpSettings *settings;
@@ -82,7 +86,7 @@ lxrdp_start(struct mod *mod, int w, int h, int bpp)
 
 /******************************************************************************/
 /* return error */
-static int DEFAULT_CC
+static int
 lxrdp_connect(struct mod *mod)
 {
     boolean ok;
@@ -189,7 +193,7 @@ lxrdp_connect(struct mod *mod)
 
 /******************************************************************************/
 /* return error */
-static int DEFAULT_CC
+static int
 lxrdp_event(struct mod *mod, int msg, long param1, long param2,
             long param3, long param4)
 {
@@ -389,7 +393,7 @@ lxrdp_event(struct mod *mod, int msg, long param1, long param2,
 
 /******************************************************************************/
 /* return error */
-static int DEFAULT_CC
+static int
 lxrdp_signal(struct mod *mod)
 {
     LLOGLN(10, ("lxrdp_signal:"));
@@ -398,7 +402,7 @@ lxrdp_signal(struct mod *mod)
 
 /******************************************************************************/
 /* return error */
-static int DEFAULT_CC
+static int
 lxrdp_end(struct mod *mod)
 {
     int i;
@@ -426,8 +430,8 @@ lxrdp_end(struct mod *mod)
 
 /******************************************************************************/
 /* return error */
-static int DEFAULT_CC
-lxrdp_set_param(struct mod *mod, const char *name, char *value)
+static int
+lxrdp_set_param(struct mod *mod, const char *name, const char *value)
 {
     rdpSettings *settings;
 
@@ -489,7 +493,7 @@ lxrdp_set_param(struct mod *mod, const char *name, char *value)
 }
 
 /******************************************************************************/
-static int DEFAULT_CC
+static int
 lxrdp_session_change(struct mod *mod, int a, int b)
 {
     LLOGLN(0, ("lxrdp_session_change: - no code here"));
@@ -497,7 +501,7 @@ lxrdp_session_change(struct mod *mod, int a, int b)
 }
 
 /******************************************************************************/
-static int DEFAULT_CC
+static int
 lxrdp_get_wait_objs(struct mod *mod, tbus *read_objs, int *rcount,
                     tbus *write_objs, int *wcount, int *timeout)
 {
@@ -520,7 +524,7 @@ lxrdp_get_wait_objs(struct mod *mod, tbus *read_objs, int *rcount,
 }
 
 /******************************************************************************/
-static int DEFAULT_CC
+static int
 lxrdp_check_wait_objs(struct mod *mod)
 {
     boolean ok;
@@ -538,7 +542,7 @@ lxrdp_check_wait_objs(struct mod *mod)
 }
 
 /******************************************************************************/
-static void DEFAULT_CC
+static void
 lfreerdp_begin_paint(rdpContext *context)
 {
     struct mod *mod;
@@ -549,7 +553,7 @@ lfreerdp_begin_paint(rdpContext *context)
 }
 
 /******************************************************************************/
-static void DEFAULT_CC
+static void
 lfreerdp_end_paint(rdpContext *context)
 {
     struct mod *mod;
@@ -560,7 +564,7 @@ lfreerdp_end_paint(rdpContext *context)
 }
 
 /******************************************************************************/
-static void DEFAULT_CC
+static void
 lfreerdp_set_bounds(rdpContext *context, rdpBounds *bounds)
 {
     struct mod *mod;
@@ -587,7 +591,7 @@ lfreerdp_set_bounds(rdpContext *context, rdpBounds *bounds)
 }
 
 /******************************************************************************/
-static void DEFAULT_CC
+static void
 lfreerdp_bitmap_update(rdpContext *context, BITMAP_UPDATE *bitmap)
 {
     struct mod *mod;
@@ -660,7 +664,7 @@ lfreerdp_bitmap_update(rdpContext *context, BITMAP_UPDATE *bitmap)
 }
 
 /******************************************************************************/
-static void DEFAULT_CC
+static void
 lfreerdp_dst_blt(rdpContext *context, DSTBLT_ORDER *dstblt)
 {
     struct mod *mod;
@@ -674,7 +678,7 @@ lfreerdp_dst_blt(rdpContext *context, DSTBLT_ORDER *dstblt)
 }
 
 /******************************************************************************/
-static void DEFAULT_CC
+static void
 lfreerdp_pat_blt(rdpContext *context, PATBLT_ORDER *patblt)
 {
     struct mod *mod;
@@ -741,7 +745,7 @@ lfreerdp_pat_blt(rdpContext *context, PATBLT_ORDER *patblt)
 }
 
 /******************************************************************************/
-static void DEFAULT_CC
+static void
 lfreerdp_scr_blt(rdpContext *context, SCRBLT_ORDER *scrblt)
 {
     struct mod *mod;
@@ -756,7 +760,7 @@ lfreerdp_scr_blt(rdpContext *context, SCRBLT_ORDER *scrblt)
 }
 
 /******************************************************************************/
-static void DEFAULT_CC
+static void
 lfreerdp_opaque_rect(rdpContext *context, OPAQUE_RECT_ORDER *opaque_rect)
 {
     struct mod *mod;
@@ -780,7 +784,7 @@ lfreerdp_opaque_rect(rdpContext *context, OPAQUE_RECT_ORDER *opaque_rect)
 }
 
 /******************************************************************************/
-static void DEFAULT_CC
+static void
 lfreerdp_mem_blt(rdpContext *context, MEMBLT_ORDER *memblt)
 {
     int id;
@@ -824,7 +828,7 @@ lfreerdp_mem_blt(rdpContext *context, MEMBLT_ORDER *memblt)
 }
 
 /******************************************************************************/
-static void DEFAULT_CC
+static void
 lfreerdp_glyph_index(rdpContext *context, GLYPH_INDEX_ORDER *glyph_index)
 {
     struct mod *mod;
@@ -885,7 +889,7 @@ lfreerdp_glyph_index(rdpContext *context, GLYPH_INDEX_ORDER *glyph_index)
 }
 
 /******************************************************************************/
-static void DEFAULT_CC
+static void
 lfreerdp_line_to(rdpContext *context, LINE_TO_ORDER *line_to)
 {
     struct mod *mod;
@@ -912,7 +916,7 @@ lfreerdp_line_to(rdpContext *context, LINE_TO_ORDER *line_to)
 }
 
 /******************************************************************************/
-static void DEFAULT_CC
+static void
 lfreerdp_cache_bitmap(rdpContext *context, CACHE_BITMAP_ORDER *cache_bitmap_order)
 {
     LLOGLN(0, ("lfreerdp_cache_bitmap: - no code here"));
@@ -920,7 +924,7 @@ lfreerdp_cache_bitmap(rdpContext *context, CACHE_BITMAP_ORDER *cache_bitmap_orde
 
 /******************************************************************************/
 /* Turn the bitmap upside down*/
-static void DEFAULT_CC
+static void
 lfreerdp_upsidedown(uint8 *destination, CACHE_BITMAP_V2_ORDER *cache_bitmap_v2_order, int server_Bpp)
 {
     tui8 *src;
@@ -947,7 +951,7 @@ lfreerdp_upsidedown(uint8 *destination, CACHE_BITMAP_V2_ORDER *cache_bitmap_v2_o
 }
 
 /******************************************************************************/
-static void DEFAULT_CC
+static void
 lfreerdp_cache_bitmapV2(rdpContext *context,
                         CACHE_BITMAP_V2_ORDER *cache_bitmap_v2_order)
 {
@@ -1030,7 +1034,7 @@ lfreerdp_cache_bitmapV2(rdpContext *context,
 }
 
 /******************************************************************************/
-static void DEFAULT_CC
+static void
 lfreerdp_cache_glyph(rdpContext *context, CACHE_GLYPH_ORDER *cache_glyph_order)
 {
     int index;
@@ -1058,7 +1062,7 @@ lfreerdp_cache_glyph(rdpContext *context, CACHE_GLYPH_ORDER *cache_glyph_order)
 }
 
 /******************************************************************************/
-static void DEFAULT_CC
+static void
 lfreerdp_cache_brush(rdpContext *context, CACHE_BRUSH_ORDER *cache_brush_order)
 {
     int idx;
@@ -1122,7 +1126,7 @@ lfreerdp_cache_brush(rdpContext *context, CACHE_BRUSH_ORDER *cache_brush_order)
 }
 
 /******************************************************************************/
-static void DEFAULT_CC
+static void
 lfreerdp_pointer_position(rdpContext *context,
                           POINTER_POSITION_UPDATE *pointer_position)
 {
@@ -1130,7 +1134,7 @@ lfreerdp_pointer_position(rdpContext *context,
 }
 
 /******************************************************************************/
-static void DEFAULT_CC
+static void
 lfreerdp_pointer_system(rdpContext *context,
                         POINTER_SYSTEM_UPDATE *pointer_system)
 {
@@ -1138,7 +1142,7 @@ lfreerdp_pointer_system(rdpContext *context,
 }
 
 /******************************************************************************/
-static void DEFAULT_CC
+static void
 lfreerdp_pointer_color(rdpContext *context,
                        POINTER_COLOR_UPDATE *pointer_color)
 {
@@ -1146,7 +1150,7 @@ lfreerdp_pointer_color(rdpContext *context,
 }
 
 /******************************************************************************/
-static int APP_CC
+static int
 lfreerdp_get_pixel(void *bits, int width, int height, int bpp,
                    int delta, int x, int y)
 {
@@ -1179,7 +1183,7 @@ lfreerdp_get_pixel(void *bits, int width, int height, int bpp,
 }
 
 /******************************************************************************/
-static int APP_CC
+static int
 lfreerdp_set_pixel(int pixel, void *bits, int width, int height, int bpp,
                    int delta, int x, int y)
 {
@@ -1225,7 +1229,7 @@ lfreerdp_set_pixel(int pixel, void *bits, int width, int height, int bpp,
 }
 
 /******************************************************************************/
-static int APP_CC
+static int
 lfreerdp_convert_color_image(void *dst, int dst_width, int dst_height,
                              int dst_bpp, int dst_delta,
                              void *src, int src_width, int src_height,
@@ -1250,7 +1254,7 @@ lfreerdp_convert_color_image(void *dst, int dst_width, int dst_height,
 }
 
 /******************************************************************************/
-static void DEFAULT_CC
+static void
 lfreerdp_pointer_new(rdpContext *context,
                      POINTER_NEW_UPDATE *pointer_new)
 {
@@ -1337,7 +1341,7 @@ lfreerdp_pointer_new(rdpContext *context,
 }
 
 /******************************************************************************/
-static void DEFAULT_CC
+static void
 lfreerdp_pointer_cached(rdpContext *context,
                         POINTER_CACHED_UPDATE *pointer_cached)
 {
@@ -1355,14 +1359,14 @@ lfreerdp_pointer_cached(rdpContext *context,
 }
 
 /******************************************************************************/
-static void DEFAULT_CC
+static void
 lfreerdp_polygon_cb(rdpContext *context, POLYGON_CB_ORDER *polygon_cb)
 {
     LLOGLN(0, ("lfreerdp_polygon_sc called:- not supported!!!!!!!!!!!!!!!!!!!!"));
 }
 
 /******************************************************************************/
-static void DEFAULT_CC
+static void
 lfreerdp_polygon_sc(rdpContext *context, POLYGON_SC_ORDER *polygon_sc)
 {
     struct mod *mod;
@@ -1417,7 +1421,7 @@ lfreerdp_polygon_sc(rdpContext *context, POLYGON_SC_ORDER *polygon_sc)
 }
 
 /******************************************************************************/
-static void DEFAULT_CC
+static void
 lfreerdp_synchronize(rdpContext *context)
 {
     struct mod *mod;
@@ -1426,7 +1430,7 @@ lfreerdp_synchronize(rdpContext *context)
 }
 
 /******************************************************************************/
-static boolean DEFAULT_CC
+static boolean
 lfreerdp_pre_connect(freerdp *instance)
 {
     struct mod *mod;
@@ -1593,7 +1597,7 @@ lfreerdp_pre_connect(freerdp *instance)
 }
 
 /*****************************************************************************/
-void DEFAULT_CC
+void
 lrail_WindowCreate(rdpContext *context, WINDOW_ORDER_INFO *orderInfo,
                    WINDOW_STATE_ORDER *window_state)
 {
@@ -1674,7 +1678,7 @@ lrail_WindowCreate(rdpContext *context, WINDOW_ORDER_INFO *orderInfo,
 }
 
 /*****************************************************************************/
-void DEFAULT_CC
+void
 lrail_WindowUpdate(rdpContext *context, WINDOW_ORDER_INFO *orderInfo,
                    WINDOW_STATE_ORDER *window_state)
 {
@@ -1683,7 +1687,7 @@ lrail_WindowUpdate(rdpContext *context, WINDOW_ORDER_INFO *orderInfo,
 }
 
 /*****************************************************************************/
-void DEFAULT_CC
+void
 lrail_WindowDelete(rdpContext *context, WINDOW_ORDER_INFO *orderInfo)
 {
     struct mod *mod;
@@ -1694,7 +1698,7 @@ lrail_WindowDelete(rdpContext *context, WINDOW_ORDER_INFO *orderInfo)
 }
 
 /*****************************************************************************/
-void DEFAULT_CC
+void
 lrail_WindowIcon(rdpContext *context, WINDOW_ORDER_INFO *orderInfo,
                  WINDOW_ICON_ORDER *window_icon)
 {
@@ -1720,7 +1724,7 @@ lrail_WindowIcon(rdpContext *context, WINDOW_ORDER_INFO *orderInfo,
 }
 
 /*****************************************************************************/
-void DEFAULT_CC
+void
 lrail_WindowCachedIcon(rdpContext *context, WINDOW_ORDER_INFO *orderInfo,
                        WINDOW_CACHED_ICON_ORDER *window_cached_icon)
 {
@@ -1735,7 +1739,7 @@ lrail_WindowCachedIcon(rdpContext *context, WINDOW_ORDER_INFO *orderInfo,
 }
 
 /*****************************************************************************/
-void DEFAULT_CC
+void
 lrail_NotifyIconCreate(rdpContext *context, WINDOW_ORDER_INFO *orderInfo,
                        NOTIFY_ICON_STATE_ORDER *notify_icon_state)
 {
@@ -1792,7 +1796,7 @@ lrail_NotifyIconCreate(rdpContext *context, WINDOW_ORDER_INFO *orderInfo,
 }
 
 /*****************************************************************************/
-void DEFAULT_CC
+void
 lrail_NotifyIconUpdate(rdpContext *context, WINDOW_ORDER_INFO *orderInfo,
                        NOTIFY_ICON_STATE_ORDER *notify_icon_state)
 {
@@ -1801,7 +1805,7 @@ lrail_NotifyIconUpdate(rdpContext *context, WINDOW_ORDER_INFO *orderInfo,
 }
 
 /*****************************************************************************/
-void DEFAULT_CC
+void
 lrail_NotifyIconDelete(rdpContext *context, WINDOW_ORDER_INFO *orderInfo)
 {
     struct mod *mod;
@@ -1813,7 +1817,7 @@ lrail_NotifyIconDelete(rdpContext *context, WINDOW_ORDER_INFO *orderInfo)
 }
 
 /*****************************************************************************/
-void DEFAULT_CC
+void
 lrail_MonitoredDesktop(rdpContext *context, WINDOW_ORDER_INFO *orderInfo,
                        MONITORED_DESKTOP_ORDER *monitored_desktop)
 {
@@ -1845,7 +1849,7 @@ lrail_MonitoredDesktop(rdpContext *context, WINDOW_ORDER_INFO *orderInfo,
 }
 
 /*****************************************************************************/
-void DEFAULT_CC
+void
 lrail_NonMonitoredDesktop(rdpContext *context, WINDOW_ORDER_INFO *orderInfo)
 {
     struct mod *mod;
@@ -1858,7 +1862,7 @@ lrail_NonMonitoredDesktop(rdpContext *context, WINDOW_ORDER_INFO *orderInfo)
 }
 
 /******************************************************************************/
-static boolean DEFAULT_CC
+static boolean
 lfreerdp_post_connect(freerdp *instance)
 {
     struct mod *mod;
@@ -1882,21 +1886,21 @@ lfreerdp_post_connect(freerdp *instance)
 }
 
 /******************************************************************************/
-static void DEFAULT_CC
+static void
 lfreerdp_context_new(freerdp *instance, rdpContext *context)
 {
     LLOGLN(0, ("lfreerdp_context_new: %p", context));
 }
 
 /******************************************************************************/
-static void DEFAULT_CC
+static void
 lfreerdp_context_free(freerdp *instance, rdpContext *context)
 {
     LLOGLN(0, ("lfreerdp_context_free: - no code here"));
 }
 
 /******************************************************************************/
-static int DEFAULT_CC
+static int
 lfreerdp_receive_channel_data(freerdp *instance, int channelId, uint8 *data,
                               int size, int flags, int total_size)
 {
@@ -1937,7 +1941,7 @@ lfreerdp_receive_channel_data(freerdp *instance, int channelId, uint8 *data,
 }
 
 /******************************************************************************/
-static boolean DEFAULT_CC
+static boolean
 lfreerdp_authenticate(freerdp *instance, char **username,
                       char **password, char **domain)
 {
@@ -1946,7 +1950,7 @@ lfreerdp_authenticate(freerdp *instance, char **username,
 }
 
 /******************************************************************************/
-static boolean DEFAULT_CC
+static boolean
 lfreerdp_verify_certificate(freerdp *instance, char *subject, char *issuer,
                             char *fingerprint)
 {
@@ -1954,6 +1958,28 @@ lfreerdp_verify_certificate(freerdp *instance, char *subject, char *issuer,
     return 1;
 }
 
+/******************************************************************************/
+static int
+lfreerdp_session_info(freerdp* instance, uint8* data, int data_bytes)
+{
+    struct mod *mod;
+    int error;
+
+    LLOGLN(10, ("lfreerdp_session_info:"));
+    error = 0;
+    mod = ((struct mod_context *)(instance->context))->modi;
+    if (mod != 0)
+    {
+        LLOGLN(10, ("lfreerdp_session_info: mod->server_session_info %p",
+               mod->server_session_info));
+        if (mod->server_session_info != 0)
+        {
+            error = mod->server_session_info(mod, (char *)data, data_bytes);
+        }
+    }
+    return error;
+}
+
 /******************************************************************************/
 tintptr EXPORT_CC
 mod_init(void)
@@ -1988,6 +2014,11 @@ mod_init(void)
     mod->inst->ReceiveChannelData = lfreerdp_receive_channel_data;
     mod->inst->Authenticate = lfreerdp_authenticate;
     mod->inst->VerifyCertificate = lfreerdp_verify_certificate;
+#if defined(VERSION_STRUCT_RDP_FREERDP)
+#if VERSION_STRUCT_RDP_FREERDP > 0
+    mod->inst->SessionInfo = lfreerdp_session_info;
+#endif
+#endif
 
     freerdp_context_new(mod->inst);
 
