@@ -21,6 +21,10 @@
  * maybe it should be called xrdp_drawable
  */
 
+#if defined(HAVE_CONFIG_H)
+#include <config_ac.h>
+#endif
+
 #include "xrdp.h"
 #include "log.h"
 #include "crc16.h"
@@ -91,7 +95,7 @@ static const unsigned int g_crc_table[256] =
 #define CRC_END(in_crc) (in_crc) = ((in_crc) ^ 0xFFFFFFFF)
 
 /*****************************************************************************/
-struct xrdp_bitmap *APP_CC
+struct xrdp_bitmap *
 xrdp_bitmap_create(int width, int height, int bpp,
                    int type, struct xrdp_wm *wm)
 {
@@ -151,7 +155,7 @@ xrdp_bitmap_create(int width, int height, int bpp,
 }
 
 /*****************************************************************************/
-struct xrdp_bitmap *APP_CC
+struct xrdp_bitmap *
 xrdp_bitmap_create_with_data(int width, int height,
                              int bpp, char *data,
                              struct xrdp_wm *wm)
@@ -202,7 +206,7 @@ xrdp_bitmap_create_with_data(int width, int height,
 }
 
 /*****************************************************************************/
-void APP_CC
+void
 xrdp_bitmap_delete(struct xrdp_bitmap *self)
 {
     int i = 0;
@@ -305,7 +309,7 @@ xrdp_bitmap_delete(struct xrdp_bitmap *self)
 }
 
 /*****************************************************************************/
-struct xrdp_bitmap *APP_CC
+struct xrdp_bitmap *
 xrdp_bitmap_get_child_by_id(struct xrdp_bitmap *self, int id)
 {
     int i = 0;
@@ -327,7 +331,7 @@ xrdp_bitmap_get_child_by_id(struct xrdp_bitmap *self, int id)
 /*****************************************************************************/
 /* if focused is true focus this window else unfocus it */
 /* returns error */
-int APP_CC
+int
 xrdp_bitmap_set_focus(struct xrdp_bitmap *self, int focused)
 {
     struct xrdp_painter *painter = (struct xrdp_painter *)NULL;
@@ -368,7 +372,7 @@ xrdp_bitmap_set_focus(struct xrdp_bitmap *self, int focused)
 }
 
 /*****************************************************************************/
-static int APP_CC
+static int
 xrdp_bitmap_get_index(struct xrdp_bitmap *self, int *palette, int color)
 {
     int r = 0;
@@ -386,7 +390,7 @@ xrdp_bitmap_get_index(struct xrdp_bitmap *self, int *palette, int color)
 
 /*****************************************************************************/
 /* returns error */
-int APP_CC
+int
 xrdp_bitmap_resize(struct xrdp_bitmap *self, int width, int height)
 {
     int Bpp = 0;
@@ -428,7 +432,7 @@ xrdp_bitmap_resize(struct xrdp_bitmap *self, int width, int height)
 /* load a bmp file */
 /* return 0 ok */
 /* return 1 error */
-int APP_CC
+int
 xrdp_bitmap_load(struct xrdp_bitmap *self, const char *filename, int *palette)
 {
     int fd = 0;
@@ -714,7 +718,7 @@ xrdp_bitmap_load(struct xrdp_bitmap *self, const char *filename, int *palette)
 }
 
 /*****************************************************************************/
-int APP_CC
+int
 xrdp_bitmap_get_pixel(struct xrdp_bitmap *self, int x, int y)
 {
     if (self == 0)
@@ -747,7 +751,7 @@ xrdp_bitmap_get_pixel(struct xrdp_bitmap *self, int x, int y)
 }
 
 /*****************************************************************************/
-int APP_CC
+int
 xrdp_bitmap_set_pixel(struct xrdp_bitmap *self, int x, int y, int pixel)
 {
     if (self == 0)
@@ -782,7 +786,7 @@ xrdp_bitmap_set_pixel(struct xrdp_bitmap *self, int x, int y, int pixel)
 /*****************************************************************************/
 /* copy part of self at x, y to 0, 0 in dest */
 /* returns error */
-int APP_CC
+int
 xrdp_bitmap_copy_box(struct xrdp_bitmap *self,
                      struct xrdp_bitmap *dest,
                      int x, int y, int cx, int cy)
@@ -898,7 +902,7 @@ xrdp_bitmap_copy_box(struct xrdp_bitmap *self,
 }
 
 /*****************************************************************************/
-int APP_CC
+int
 xrdp_bitmap_hash_crc(struct xrdp_bitmap *self)
 {
     void *hash;
@@ -945,7 +949,7 @@ xrdp_bitmap_hash_crc(struct xrdp_bitmap *self)
 /*****************************************************************************/
 /* copy part of self at x, y to 0, 0 in dest */
 /* returns error */
-int APP_CC
+int
 xrdp_bitmap_copy_box_with_crc(struct xrdp_bitmap *self,
                               struct xrdp_bitmap *dest,
                               int x, int y, int cx, int cy)
@@ -1208,7 +1212,7 @@ xrdp_bitmap_copy_box_with_crc(struct xrdp_bitmap *self,
 
 /*****************************************************************************/
 /* returns true if they are the same, else returns false */
-int APP_CC
+int
 xrdp_bitmap_compare(struct xrdp_bitmap *self,
                     struct xrdp_bitmap *b)
 {
@@ -1248,7 +1252,7 @@ xrdp_bitmap_compare(struct xrdp_bitmap *self,
 }
 
 /*****************************************************************************/
-static int APP_CC
+static int
 xrdp_bitmap_draw_focus_box(struct xrdp_bitmap *self,
                            struct xrdp_painter *painter,
                            int x, int y, int cx, int cy)
@@ -1286,7 +1290,7 @@ xrdp_bitmap_draw_focus_box(struct xrdp_bitmap *self,
 
 /*****************************************************************************/
 /* x and y are in relation to self for 0, 0 is the top left of the control */
-static int APP_CC
+static int
 xrdp_bitmap_draw_button(struct xrdp_bitmap *self,
                         struct xrdp_painter *painter,
                         int x, int y, int w, int h,
@@ -1353,7 +1357,7 @@ xrdp_bitmap_draw_button(struct xrdp_bitmap *self,
 /*****************************************************************************/
 /* nil for rect means the whole thing */
 /* returns error */
-int APP_CC
+int
 xrdp_bitmap_invalidate(struct xrdp_bitmap *self, struct xrdp_rect *rect)
 {
     int i;
@@ -1764,7 +1768,7 @@ xrdp_bitmap_invalidate(struct xrdp_bitmap *self, struct xrdp_rect *rect)
 
 /*****************************************************************************/
 /* returns error */
-int APP_CC
+int
 xrdp_bitmap_def_proc(struct xrdp_bitmap *self, int msg,
                      int param1, int param2)
 {
@@ -2069,7 +2073,7 @@ xrdp_bitmap_def_proc(struct xrdp_bitmap *self, int msg,
 
 /*****************************************************************************/
 /* convert the controls coords to screen coords */
-int APP_CC
+int
 xrdp_bitmap_to_screenx(struct xrdp_bitmap *self, int x)
 {
     int i;
@@ -2087,7 +2091,7 @@ xrdp_bitmap_to_screenx(struct xrdp_bitmap *self, int x)
 
 /*****************************************************************************/
 /* convert the controls coords to screen coords */
-int APP_CC
+int
 xrdp_bitmap_to_screeny(struct xrdp_bitmap *self, int y)
 {
     int i;
@@ -2105,7 +2109,7 @@ xrdp_bitmap_to_screeny(struct xrdp_bitmap *self, int y)
 
 /*****************************************************************************/
 /* convert the screen coords to controls coords */
-int APP_CC
+int
 xrdp_bitmap_from_screenx(struct xrdp_bitmap *self, int x)
 {
     int i;
@@ -2123,7 +2127,7 @@ xrdp_bitmap_from_screenx(struct xrdp_bitmap *self, int x)
 
 /*****************************************************************************/
 /* convert the screen coords to controls coords */
-int APP_CC
+int
 xrdp_bitmap_from_screeny(struct xrdp_bitmap *self, int y)
 {
     int i;
@@ -2140,7 +2144,7 @@ xrdp_bitmap_from_screeny(struct xrdp_bitmap *self, int y)
 }
 
 /*****************************************************************************/
-int APP_CC
+int
 xrdp_bitmap_get_screen_clip(struct xrdp_bitmap *self,
                             struct xrdp_painter *painter,
                             struct xrdp_rect *rect,
