@@ -18,13 +18,17 @@
  * simple window manager
  */
 
+#if defined(HAVE_CONFIG_H)
+#include <config_ac.h>
+#endif
+
 #include <stdarg.h>
 #include <stdio.h>
 #include "xrdp.h"
 #include "log.h"
 
 /*****************************************************************************/
-struct xrdp_wm *APP_CC
+struct xrdp_wm *
 xrdp_wm_create(struct xrdp_process *owner,
                struct xrdp_client_info *client_info)
 {
@@ -68,7 +72,7 @@ xrdp_wm_create(struct xrdp_process *owner,
 }
 
 /*****************************************************************************/
-void APP_CC
+void
 xrdp_wm_delete(struct xrdp_wm *self)
 {
     if (self == 0)
@@ -94,21 +98,21 @@ xrdp_wm_delete(struct xrdp_wm *self)
 }
 
 /*****************************************************************************/
-int APP_CC
+int
 xrdp_wm_send_palette(struct xrdp_wm *self)
 {
     return libxrdp_send_palette(self->session, self->palette);
 }
 
 /*****************************************************************************/
-int APP_CC
+int
 xrdp_wm_send_bell(struct xrdp_wm *self)
 {
     return libxrdp_send_bell(self->session);
 }
 
 /*****************************************************************************/
-int APP_CC
+int
 xrdp_wm_send_bitmap(struct xrdp_wm *self, struct xrdp_bitmap *bitmap,
                     int x, int y, int cx, int cy)
 {
@@ -117,7 +121,7 @@ xrdp_wm_send_bitmap(struct xrdp_wm *self, struct xrdp_bitmap *bitmap,
 }
 
 /*****************************************************************************/
-int APP_CC
+int
 xrdp_wm_set_focused(struct xrdp_wm *self, struct xrdp_bitmap *wnd)
 {
     struct xrdp_bitmap *focus_out_control;
@@ -156,7 +160,7 @@ xrdp_wm_set_focused(struct xrdp_wm *self, struct xrdp_bitmap *wnd)
 }
 
 /******************************************************************************/
-static int APP_CC
+static int
 xrdp_wm_get_pixel(char *data, int x, int y, int width, int bpp)
 {
     int start;
@@ -189,7 +193,7 @@ xrdp_wm_get_pixel(char *data, int x, int y, int width, int bpp)
 }
 
 /*****************************************************************************/
-int APP_CC
+int
 xrdp_wm_pointer(struct xrdp_wm *self, char *data, char *mask, int x, int y,
                 int bpp)
 {
@@ -213,7 +217,7 @@ xrdp_wm_pointer(struct xrdp_wm *self, char *data, char *mask, int x, int y,
 
 /*****************************************************************************/
 /* returns error */
-int APP_CC
+int
 xrdp_wm_load_pointer(struct xrdp_wm *self, char *file_name, char *data,
                      char *mask, int *x, int *y)
 {
@@ -315,7 +319,7 @@ xrdp_wm_load_pointer(struct xrdp_wm *self, char *file_name, char *data,
 }
 
 /*****************************************************************************/
-int APP_CC
+int
 xrdp_wm_send_pointer(struct xrdp_wm *self, int cache_idx,
                      char *data, char *mask, int x, int y, int bpp)
 {
@@ -324,7 +328,7 @@ xrdp_wm_send_pointer(struct xrdp_wm *self, int cache_idx,
 }
 
 /*****************************************************************************/
-int APP_CC
+int
 xrdp_wm_set_pointer(struct xrdp_wm *self, int cache_idx)
 {
     return libxrdp_set_pointer(self->session, cache_idx);
@@ -332,7 +336,7 @@ xrdp_wm_set_pointer(struct xrdp_wm *self, int cache_idx)
 
 /*****************************************************************************/
 /* convert hex string to int */
-unsigned int APP_CC
+unsigned int
 xrdp_wm_htoi (const char *ptr)
 {
     unsigned int value = 0;
@@ -367,7 +371,7 @@ xrdp_wm_htoi (const char *ptr)
 }
 
 /*****************************************************************************/
-int APP_CC
+int
 xrdp_wm_load_static_colors_plus(struct xrdp_wm *self, char *autorun_name)
 {
     int bindex;
@@ -518,7 +522,7 @@ xrdp_wm_load_static_colors_plus(struct xrdp_wm *self, char *autorun_name)
 
 /*****************************************************************************/
 /* returns error */
-int APP_CC
+int
 xrdp_wm_load_static_pointers(struct xrdp_wm *self)
 {
     struct xrdp_pointer_item pointer_item;
@@ -540,7 +544,7 @@ xrdp_wm_load_static_pointers(struct xrdp_wm *self)
 }
 
 /*****************************************************************************/
-int APP_CC
+int
 xrdp_wm_init(struct xrdp_wm *self)
 {
     int fd;
@@ -550,6 +554,7 @@ xrdp_wm_init(struct xrdp_wm *self)
     char *q;
     const char *r;
     char param[256];
+    char default_section_name[256];
     char section_name[256];
     char cfg_file[256];
     char autorun_name[256];
@@ -562,7 +567,7 @@ xrdp_wm_init(struct xrdp_wm *self)
     xrdp_wm_load_static_pointers(self);
     self->screen->bg_color = self->xrdp_config->cfg_globals.ls_top_window_bg_color;
 
-    if (self->session->client_info->rdp_autologin || self->hide_log_window)
+    if (self->session->client_info->rdp_autologin)
     {
         /*
          * NOTE: this should eventually be accessed from self->xrdp_config
@@ -577,6 +582,22 @@ xrdp_wm_init(struct xrdp_wm *self)
             values = list_create();
             values->auto_free = 1;
 
+            /* pick up the first section name except for 'globals', 'Logging', 'channels'
+             * in xrdp.ini and use it as default section name */
+            file_read_sections(fd, names);
+            default_section_name[0] = '\0';
+            for (index = 0; index < names->count; index++)
+            {
+                q = (char *)list_get_item(names, index);
+                if ((g_strncasecmp("globals", q, 8) != 0) &&
+                    (g_strncasecmp("Logging", q, 8) != 0) &&
+                    (g_strncasecmp("channels", q, 9) != 0))
+                {
+                    g_strncpy(default_section_name, q, 255);
+                    break;
+                }
+            }
+
             /* look for module name to be loaded */
             if (autorun_name[0] != 0)
             {
@@ -590,33 +611,36 @@ xrdp_wm_init(struct xrdp_wm *self)
                  * simplify for the user in a proxy setup */
 
                 /* we use the domain name as the module name to be loaded */
-                g_strncpy(section_name, self->session->client_info->domain,
-                              255);
+                g_strncpy(section_name,
+                          self->session->client_info->domain, 255);
             }
             else
             {
-                /* if no domain is passed, and no autorun in xrdp.ini,
-                   use the first item in the xrdp.ini
-                   file that's not named
-                   'globals' or 'Logging' or 'channels' */
-                /* TODO: change this and have an 'autologin'
-                   line in globals section */
-                file_read_sections(fd, names);
-                for (index = 0; index < names->count; index++)
-                {
-                    q = (char *)list_get_item(names, index);
-                    if ((g_strncasecmp("globals", q, 8) != 0) &&
-                        (g_strncasecmp("Logging", q, 8) != 0) &&
-                        (g_strncasecmp("channels", q, 9) != 0))
-                    {
-                        g_strncpy(section_name, q, 255);
-                        break;
-                    }
-                }
+                /* if no domain is given, and autorun is not specified in xrdp.ini
+                 * use default_section_name as section_name  */
+                g_strncpy(section_name, default_section_name, 255);
             }
 
             list_clear(names);
 
+            /* if given section name doesn't match any sections configured
+             * in xrdp.ini, fallback to default_section_name */
+            if (file_read_section(fd, section_name, names, values) != 0)
+            {
+                log_message(LOG_LEVEL_INFO,
+                            "Module \"%s\" specified by %s from %s port %s "
+                            "is not configured. Using \"%s\" instead.",
+                            section_name,
+                            self->session->client_info->username,
+                            self->session->client_info->client_addr,
+                            self->session->client_info->client_port,
+                            default_section_name);
+                list_clear(names);
+                list_clear(values);
+
+                g_strncpy(section_name, default_section_name, 255);
+            }
+
             /* look for the required module in xrdp.ini, fetch its parameters */
             if (file_read_section(fd, section_name, names, values) == 0)
             {
@@ -679,10 +703,9 @@ xrdp_wm_init(struct xrdp_wm *self)
             }
             else
             {
-                /* requested module name not found in xrdp.ini */
-                xrdp_wm_log_msg(self, LOG_LEVEL_ERROR,
-                                "Section \"%s\" not configured in xrdp.ini",
-                                section_name);
+                /* Hopefully, we never reach here. */
+                log_message(LOG_LEVEL_DEBUG,
+                            "Control should never reach %s:%d", __FILE__, __LINE__);
             }
 
             list_delete(names);
@@ -711,7 +734,7 @@ xrdp_wm_init(struct xrdp_wm *self)
 /*****************************************************************************/
 /* returns the number for rects visible for an area relative to a drawable */
 /* putting the rects in region */
-int APP_CC
+int
 xrdp_wm_get_vis_region(struct xrdp_wm *self, struct xrdp_bitmap *bitmap,
                        int x, int y, int cx, int cy,
                        struct xrdp_region *region, int clip_children)
@@ -759,7 +782,7 @@ xrdp_wm_get_vis_region(struct xrdp_wm *self, struct xrdp_bitmap *bitmap,
 
 /*****************************************************************************/
 /* return the window at x, y on the screen */
-static struct xrdp_bitmap *APP_CC
+static struct xrdp_bitmap *
 xrdp_wm_at_pos(struct xrdp_bitmap *wnd, int x, int y,
                struct xrdp_bitmap **wnd1)
 {
@@ -797,7 +820,7 @@ xrdp_wm_at_pos(struct xrdp_bitmap *wnd, int x, int y,
 }
 
 /*****************************************************************************/
-static int APP_CC
+static int
 xrdp_wm_xor_pat(struct xrdp_wm *self, int x, int y, int cx, int cy)
 {
     self->painter->clip_children = 0;
@@ -837,7 +860,7 @@ xrdp_wm_xor_pat(struct xrdp_wm *self, int x, int y, int cx, int cy)
 /*****************************************************************************/
 /* return true if rect is totally exposed going in reverse z order */
 /* from wnd up */
-static int APP_CC
+static int
 xrdp_wm_is_rect_vis(struct xrdp_wm *self, struct xrdp_bitmap *wnd,
                     struct xrdp_rect *rect)
 {
@@ -886,7 +909,7 @@ xrdp_wm_is_rect_vis(struct xrdp_wm *self, struct xrdp_bitmap *wnd,
 }
 
 /*****************************************************************************/
-static int APP_CC
+static int
 xrdp_wm_move_window(struct xrdp_wm *self, struct xrdp_bitmap *wnd,
                     int dx, int dy)
 {
@@ -941,7 +964,7 @@ xrdp_wm_move_window(struct xrdp_wm *self, struct xrdp_bitmap *wnd,
 
 
 /*****************************************************************************/
-static int APP_CC
+static int
 xrdp_wm_undraw_dragging_box(struct xrdp_wm *self, int do_begin_end)
 {
     int boxx;
@@ -977,7 +1000,7 @@ xrdp_wm_undraw_dragging_box(struct xrdp_wm *self, int do_begin_end)
 }
 
 /*****************************************************************************/
-static int APP_CC
+static int
 xrdp_wm_draw_dragging_box(struct xrdp_wm *self, int do_begin_end)
 {
     int boxx;
@@ -1013,7 +1036,7 @@ xrdp_wm_draw_dragging_box(struct xrdp_wm *self, int do_begin_end)
 }
 
 /*****************************************************************************/
-int APP_CC
+int
 xrdp_wm_mouse_move(struct xrdp_wm *self, int x, int y)
 {
     struct xrdp_bitmap *b;
@@ -1118,7 +1141,7 @@ xrdp_wm_mouse_move(struct xrdp_wm *self, int x, int y)
 }
 
 /*****************************************************************************/
-static int APP_CC
+static int
 xrdp_wm_clear_popup(struct xrdp_wm *self)
 {
     int i;
@@ -1142,7 +1165,7 @@ xrdp_wm_clear_popup(struct xrdp_wm *self)
 }
 
 /*****************************************************************************/
-int APP_CC
+int
 xrdp_wm_mouse_click(struct xrdp_wm *self, int x, int y, int but, int down)
 {
     struct xrdp_bitmap *control;
@@ -1210,6 +1233,10 @@ xrdp_wm_mouse_click(struct xrdp_wm *self, int x, int y, int but, int down)
         {
             if (self->mm->mod->mod_event != 0)
             {
+                if (down)
+                {
+                    self->mm->mod->mod_event(self->mm->mod, WM_MOUSEMOVE, x, y, 0, 0);
+                }
                 if (but == 1 && down)
                 {
                     self->mm->mod->mod_event(self->mm->mod, WM_LBUTTONDOWN, x, y, 0, 0);
@@ -1394,7 +1421,7 @@ xrdp_wm_mouse_click(struct xrdp_wm *self, int x, int y, int but, int down)
 }
 
 /*****************************************************************************/
-int APP_CC
+int
 xrdp_wm_key(struct xrdp_wm *self, int device_flags, int scan_code)
 {
     int msg;
@@ -1460,7 +1487,7 @@ xrdp_wm_key(struct xrdp_wm *self, int device_flags, int scan_code)
 
 /*****************************************************************************/
 /* happens when client gets focus and sends key modifier info */
-int APP_CC
+int
 xrdp_wm_key_sync(struct xrdp_wm *self, int device_flags, int key_flags)
 {
     self->num_lock = 0;
@@ -1495,7 +1522,7 @@ xrdp_wm_key_sync(struct xrdp_wm *self, int device_flags, int key_flags)
 }
 
 /*****************************************************************************/
-int APP_CC
+int
 xrdp_wm_key_unicode(struct xrdp_wm *self, int device_flags, int unicode)
 {
     int index;
@@ -1572,7 +1599,7 @@ xrdp_wm_key_unicode(struct xrdp_wm *self, int device_flags, int unicode)
 }
 
 /*****************************************************************************/
-int APP_CC
+int
 xrdp_wm_pu(struct xrdp_wm *self, struct xrdp_bitmap *control)
 {
     int x;
@@ -1605,7 +1632,7 @@ xrdp_wm_pu(struct xrdp_wm *self, struct xrdp_bitmap *control)
 }
 
 /*****************************************************************************/
-static int APP_CC
+static int
 xrdp_wm_process_input_mouse(struct xrdp_wm *self, int device_flags,
                             int x, int y)
 {
@@ -1668,7 +1695,7 @@ xrdp_wm_process_input_mouse(struct xrdp_wm *self, int device_flags,
 }
 
 /*****************************************************************************/
-static int APP_CC
+static int
 xrdp_wm_process_input_mousex(struct xrdp_wm* self, int device_flags,
                              int x, int y)
 {
@@ -1702,7 +1729,7 @@ xrdp_wm_process_input_mousex(struct xrdp_wm* self, int device_flags,
    param2 = size
    param3 = pointer to data
    param4 = total size */
-static int APP_CC
+static int
 xrdp_wm_process_channel_data(struct xrdp_wm *self,
                              tbus param1, tbus param2,
                              tbus param3, tbus param4)
@@ -1738,8 +1765,9 @@ xrdp_wm_process_channel_data(struct xrdp_wm *self,
 
 /******************************************************************************/
 /* this is the callbacks coming from libxrdp.so */
-int DEFAULT_CC
-callback(long id, int msg, long param1, long param2, long param3, long param4)
+int
+callback(intptr_t id, int msg, intptr_t param1, intptr_t param2,
+         intptr_t param3, intptr_t param4)
 {
     int rv;
     struct xrdp_wm *wm;
@@ -1800,7 +1828,7 @@ callback(long id, int msg, long param1, long param2, long param3, long param4)
 /******************************************************************************/
 /* returns error */
 /* this gets called when there is nothing on any socket */
-static int APP_CC
+static int
 xrdp_wm_login_mode_changed(struct xrdp_wm *self)
 {
     if (self == 0)
@@ -1844,7 +1872,7 @@ xrdp_wm_login_mode_changed(struct xrdp_wm *self)
 
 /*****************************************************************************/
 /* this is the log windows notify function */
-static int DEFAULT_CC
+static int
 xrdp_wm_log_wnd_notify(struct xrdp_bitmap *wnd,
                        struct xrdp_bitmap *sender,
                        int msg, long param1, long param2)
@@ -1927,7 +1955,7 @@ add_string_to_logwindow(const char *msg, struct list *log)
 }
 
 /*****************************************************************************/
-int APP_CC
+int
 xrdp_wm_show_log(struct xrdp_wm *self)
 {
     struct xrdp_bitmap *but;
@@ -2016,7 +2044,7 @@ xrdp_wm_show_log(struct xrdp_wm *self)
 }
 
 /*****************************************************************************/
-int APP_CC
+int
 xrdp_wm_log_msg(struct xrdp_wm *self, enum logLevels loglevel,
                 const char *fmt, ...)
 {
@@ -2033,7 +2061,7 @@ xrdp_wm_log_msg(struct xrdp_wm *self, enum logLevels loglevel,
 }
 
 /*****************************************************************************/
-int APP_CC
+int
 xrdp_wm_get_wait_objs(struct xrdp_wm *self, tbus *robjs, int *rc,
                       tbus *wobjs, int *wc, int *timeout)
 {
@@ -2051,7 +2079,7 @@ xrdp_wm_get_wait_objs(struct xrdp_wm *self, tbus *robjs, int *rc,
 }
 
 /******************************************************************************/
-int APP_CC
+int
 xrdp_wm_check_wait_objs(struct xrdp_wm *self)
 {
     int rv;
@@ -2078,7 +2106,7 @@ xrdp_wm_check_wait_objs(struct xrdp_wm *self)
 }
 
 /*****************************************************************************/
-int APP_CC
+int
 xrdp_wm_set_login_mode(struct xrdp_wm *self, int login_mode)
 {
     self->login_mode = login_mode;
